package com.hms.bean;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.system.CapturedOutput;
import org.springframework.boot.test.system.OutputCaptureExtension;
import org.springframework.context.ApplicationContext;

import static org.assertj.core.api.Assertions.assertThat;

@SpringBootTest
@ExtendWith(OutputCaptureExtension.class)
public class BeanCreationTest {

    @Autowired
    ApplicationContext context;

    /**
     * When the listener is eagerly created, then the @EventListener annotation works fine.
     * When the listener is created on request (lazily), then the annotation doesn't work.
     */
    @Test
    public void annotationShouldWork(CapturedOutput output) {
        final String msg = "Annotation Should Work";
        IEventListener eventListener = context.getBean(IEventListener.class);
        assertThat(eventListener).isInstanceOf(EventListenerImpl.class);
        context.publishEvent(new StringEvent(msg));
        assertThat(output.getOut()).contains(msg);
    }
}
